@@ -26,8 +26,7 @@ function [f,p,cvg,iter,corp,covp,covr,stdresid,Z,r2]= ...
   %%
   %% Version 3.beta
   %% Optional parameters are in braces {}.
-  %% x = vector or matrix of independent variables, 1 entry or row per
-  %%   observation.
+  %% x = vector or matrix of independent variables.
   %% y = vector or matrix of observed values.
   %% wt = statistical weights (same dimensions as y).  These should be
   %%   set to be proportional to (sqrt of var(y))^-1; (That is, the
@@ -563,6 +562,7 @@ function [f,p,cvg,iter,corp,covp,covr,stdresid,Z,r2]= ...
   hook.fixed = dp == 0;
   if (verbose)
     hook.Display = 'iter';
+    __plot_cmds__ = @ __plot_cmds__; # for bug #31484 (Octave <= 3.2.4)
     hook.plot_cmd = @ (f) __plot_cmds__ (x, y, y - f);
   else
     hook.Display = 'off';
